@@ -100,7 +100,7 @@ char *print_model_options(options_t opt, char *buf) {
   strcat(buf, MOPTION(M_OPTION_SHOW_SCRIPT, opt));
   strcat(buf, MOPTION(M_OPTION_SHOW_STYLE, opt));
   strcat(buf, MOPTION(M_OPTION_SHOW_HTML_COMMENTS, opt));
-  strcat(buf, MOPTION(M_OPTION_USE_REGEX, opt));
+  strcat(buf, MOPTION(M_OPTION_USE_STDTOK, opt));
   strcat(buf, MOPTION(M_OPTION_ATTACHMENTS, opt));
   strcat(buf, MOPTION(M_OPTION_WARNING_BAD, opt));
   strcat(buf, MOPTION(M_OPTION_NGRAM_STRADDLE_NL, opt));
@@ -123,7 +123,7 @@ char *print_user_options(options_t opt, char *buf) {
   strcat(buf, MOPTION(U_OPTION_DECIMATE, opt));
   strcat(buf, MOPTION(U_OPTION_GROWHASH, opt));
   strcat(buf, MOPTION(U_OPTION_INDENTED, opt));
-  strcat(buf, MOPTION(U_OPTION_ZEROLEARN, opt));
+  strcat(buf, MOPTION(U_OPTION_NOZEROLEARN, opt));
   strcat(buf, MOPTION(U_OPTION_LAPLACE, opt));
   strcat(buf, MOPTION(U_OPTION_DIRICHLET, opt));
   strcat(buf, MOPTION(U_OPTION_JAYNES, opt));
@@ -154,6 +154,7 @@ error_code_t sanitize_model_options(options_t *mopt, category_t *cat) {
     (1<<M_OPTION_REFMODEL)|
     (1<<M_OPTION_I18N)|
     (1<<M_OPTION_CASEN)|
+    (1<<M_OPTION_USE_STDTOK)|
     (1<<M_OPTION_CALCENTROPY)|
     (1<<M_OPTION_MULTINOMIAL);
   *mopt |= (cat->m_options & mask);
@@ -216,7 +217,7 @@ error_code_t sanitize_model_options(options_t *mopt, category_t *cat) {
     (1<<M_OPTION_CHAR_CEF);
 
   if( (*mopt & mask) && ((cat->m_options & mask) != (*mopt & mask)) ) {
-    errormsg(E_ERROR,
+    errormsg(E_FATAL,
 	    "category %s has incompatible token set (check -e switch)\n",
 	    cat->filename);
     return 0;
@@ -267,7 +268,23 @@ regex_count_t load_regex(char *buf) {
   }
   if( c >= regex_count ) { /* not found */
     /* add it to our list */
-    re[regex_count].string = strdup(buf);
+    if( strchr(buf, '(') ) {
+      re[regex_count].string = strdup(buf);
+    } else {
+      char *dup = (char *)malloc(strlen(buf)+2);
+      if( dup ) {
+	sprintf(dup, "(%s)", buf);
+	errormsg(E_WARNING,
+		 "no captures found in regex, converting to '%s'\n",
+		 dup);
+      }
+      re[regex_count].string = dup;
+    }
+    if( !re[regex_count].string ) {
+      errormsg(E_FATAL,
+	       "could not prepare regular expression '%s'.\n",
+	       buf);
+    }
     /* and compile the regex */
     if( regcomp(&re[regex_count].regex, 
 		re[regex_count].string, REG_EXTENDED) != 0 ) {
@@ -415,7 +432,8 @@ void init_category(category_t *cat) {
     cat->beta = 0.0;
     cat->mu = 0.0;
     cat->s2 = 0.0;
-    cat->complexity = 0;
+    cat->complexity = 0.0;
+    cat->fcomplexity = 0;
     cat->max_order = 0;
     p = strrchr(cat->fullfilename, '/');
     if( p ) {
@@ -434,16 +452,74 @@ void init_category(category_t *cat) {
     cat->c_options = 0;
     cat->hash = NULL;
     cat->mmap_offset = 0;
-    cat->mmap_start = 0;
+    cat->mmap_start = NULL;
 }
 
-/* frees the resrouces associated with a category */
-void free_category(category_t *cat) {
+bool_t create_category_hash(category_t *cat, FILE *input, int protf) {
+  hash_count_t i, j;
+
+  if( u_options & (1<<U_OPTION_MMAP) ) {
+    cat->mmap_offset = ftell(input);
+    if( cat->mmap_offset > 0 ) {
+      cat->mmap_start = 
+	(byte_t *)MMAP(0, sizeof(c_item_t) * cat->max_tokens + 
+		       cat->mmap_offset,
+		       protf, MAP_SHARED, fileno(input), 0);
+      if( cat->mmap_start ) {
+	cat->hash = (c_item_t *)(cat->mmap_start + cat->mmap_offset);
+	MADVISE(cat->hash, sizeof(c_item_t) * cat->max_tokens, 
+		MADV_SEQUENTIAL|MADV_WILLNEED);
+	/* lock the pages to prevent swapping - on Linux, this
+	   works without root privs so long as the user limits
+	   are big enough - mine are unlimited ;-) 
+	   On other OSes, root may me necessary. If we can't
+	   lock, it doesn't really matter, but cross validations
+	   and multiple classifications are a _lot_ faster with locking. */
+	MLOCK(cat->hash, sizeof(c_item_t) * cat->max_tokens);
+	cat->c_options |= (1<<C_OPTION_MMAPPED_HASH);
+      }
+    }
+  }
+
+  if( !cat->hash ) {
+    cat->c_options &= ~(1<<C_OPTION_MMAPPED_HASH);
+    /* allocate hash table */
+    cat->hash = (c_item_t *)malloc(sizeof(c_item_t) * cat->max_tokens);
+    if( !cat->hash ) {
+      errormsg(E_ERROR, "not enough memory for category %s\n", 
+	       cat->filename);
+      return 0;
+    }
+
+    MADVISE(cat->hash, sizeof(c_item_t) * cat->max_tokens, 
+	    MADV_SEQUENTIAL);
+
+    /* read in hash table */
+    i = cat->max_tokens;
+    j = 0;
+    while(!ferror(input) && !feof(input) && (j < i) ) {
+      j += fread(cat->hash + j, sizeof(c_item_t), i - j, input);
+    }
+
+    if( j < i ) {
+      errormsg(E_ERROR, "corrupt category? %s\n",
+	       cat->fullfilename);
+      free(cat->hash);
+      return 0;
+    }
+
+  }
+  return 1;
+}
+
+
+void free_category_hash(category_t *cat) {
   if( cat->hash ) {
-    if( cat->c_options & (1<<C_OPTION_MMAPPED_HASH) ) {
+    if( cat->mmap_start != NULL ) {
       MUNMAP(cat->mmap_start, cat->max_tokens * sizeof(c_item_t) + 
 	     cat->mmap_offset);
       cat->mmap_start = NULL;
+      cat->mmap_offset = 0;
       cat->hash = NULL;
     }
     if( cat->hash ) {
@@ -451,6 +527,11 @@ void free_category(category_t *cat) {
       cat->hash = NULL;
     }
   }
+}
+
+/* frees the resrouces associated with a category */
+void free_category(category_t *cat) {
+  free_category_hash(cat);
   if( cat->filename ) { free(cat->filename); }
   if( cat->fullfilename ) { free(cat->fullfilename); }
 }
@@ -495,7 +576,12 @@ void init_purely_random_text_category(category_t *cat) {
 #endif
 
   cat->logZ = 0.0;
+  cat->divergence = 0.0;
+  cat->delta = 0.0;
+  cat->renorm = 0.0;
   cat->hash = NULL;
+  cat->mmap_start = NULL;
+  cat->mmap_offset = 0;
   cat->model_type = simple;
   cat->max_order = 1;
   cat->m_options = 0;
@@ -533,7 +619,8 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
   category_count_t i = 0;
   weight_t multinomial_correction = 0.0;
   weight_t shannon_correction = 0.0;
-  weight_t renorm, lambda, ref, oldscore;
+  weight_t lambda, ref, oldscore;
+  bool_t apply;
   alphabet_size_t pp, pc;
   hash_value_t id;
   char *q;
@@ -612,11 +699,18 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
       oldscore = cat[i].score;
       lambda = 0.0;
       ref = 0.0;
-      renorm = 0.0;
 
-      /* see if this is for us */
-      if( ((re == 0) && (tt.order <= cat[i].max_order)) ||
-	    ((re > 0) && (cat[i].retype & (1<<(re-1)))) ) {
+      /* see if this token is for us. The rule is: a category either
+	 uses the standard tokenizer (in that case re = INVALID_RE),
+	 or it uses only those regexes which are listed in the retype
+	 bitmap. Since re = 0 is taken by the standard tokenizer, 
+	 this occurs when re > 0 and we have to subtract 1 to check
+	 the bitmap. Simple, really ;-) */
+      apply = ( ((re == INVALID_RE) && 
+		 (tt.order <= cat[i].max_order) && !cat[i].retype) ||
+		((re > 0) && 
+		 (cat[i].retype & (1<<(re-1)))) );
+      if( apply ) {
 
 	/* if token found, add its lambda weight */
 	k = find_in_category(&cat[i], id);
@@ -641,26 +735,75 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
 	  }
 	  ref = UNPACK_RWEIGHTS(PACK_RWEIGHTS(ref));
 
-	  /* don't forget the normalizing constant, and update
-             complexity for this category - note that by updating the
-             complexity on order 1 tokens instead of max_order tokens,
-             we slightly overestimate (by max_order - 1), but a
-             benefit is that we start off well away from zero, instead
-             of hovering above and below zero. */
-	  renorm = cat[i].logZ;      
-	  cat[i].complexity++;
 	}
 
+	/* update the complexity */
+	/* this is actually very simple in hindsight, but took
+	   me a long time to get right. Different versions of dbacl
+	   compute the complexity in different ways, and I kept changing
+	   the method because I wasn't happy. 
+
+	   In previous versions, complexity is an integer, which begs
+	   the question "what does it count?".  For simple models
+	   (max_order = 1) this is easy: we count the number of
+	   tokens. But for max_order > 1, it's not obvious, because we
+	   need to divide by 1/max_order asymptotically.
+
+	   One way is to increment the complexity if we encounter a
+	   token of order max_order. This is correct for Markovian
+	   models and corresponds to the dbacl.ps writeup, but causes
+	   trouble in some edge cases. For example, if we classify a
+	   very short document, there might not be enough tokens to
+	   make sense.  This actually occurs when dbacl must classify
+	   individual lines, and some lines contain one or two tokens
+	   only.  Worse, dbacl used to renormalize at the same time as
+	   updating the complexity, which increases the likelihood of
+	   having a negative divergence score estimate in the first
+	   few iterations - very bad.  Finally, the complexity is
+	   nearly meaningless for models built up from regular
+	   expressions, because both the start and the end of each
+	   line contains incomplete n-grams (recall regexes can't
+	   straddle newlines).
+
+	   So to solve these problems, some previous versions of dbacl
+	   counted always the order 1 tokens. Asymptotically, this
+	   makes no difference, but again it fails on edge
+	   cases. Firstly, doing this means that the complexity for a
+	   simple model is the same as the complexity for an n-gram
+	   model for any n, so that makes it hard to compare mixed
+	   models because n-gram model scores are consitently biased
+	   for n > 1. Another problem is again with regexes, because
+	   the incomplete n-gram tokens at the start and end of each
+	   line add up to a pretty large error over thousands of
+	   tokens.
+
+	   The solution to the above problems is twofold: first, we
+	   renormalize after each token, regardless of its order. Of
+	   course this means we must divide logZ by the number of
+	   tokens per complexity unit, ie renorm = delta * logZ with
+	   delta = 1/max_order. Once I realized this it was obvious
+	   that the complexity should be also incremented by delta for
+	   every token. As a side effect, the complexity is now a real
+	   number, and actually measures not just the max_order token
+	   count, but also the fraction of incomplete n-grams. This
+	   seems like the right way to go, especially for models based
+	   on regexes, since now we also count the incomplete n-grams
+	   at both ends of the line, which adds up to quite a bit over
+	   many lines. */
+
+	cat[i].fcomplexity++; /* don't actually need this, but nice to have */
+	cat[i].complexity += cat[i].delta;
+
 	/* now adjust the score */
 	switch(cat[i].model_type) {
 	case simple:
 	  multinomial_correction = h ?
 	    (log((weight_t)cat[i].complexity) - log((weight_t)h->count)) : 0.0;
-	  cat[i].score += lambda + multinomial_correction + ref - renorm;
+	  cat[i].score += lambda + multinomial_correction + ref - cat[i].renorm;
 	  break;
 	case sequential:
 	default:
-	  cat[i].score += lambda + ref - renorm;
+	  cat[i].score += lambda + ref - cat[i].renorm;
 	  if( tt.order == cat[i].max_order ) {
 	    cat[i].score_shannon += shannon_correction;
 	  }
@@ -676,7 +819,7 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
 
       if( u_options & (1<<U_OPTION_DUMP) ) {
 	if( u_options & (1<<U_OPTION_SCORES) ) {
-	  fprintf(stdout, " %8.2f * %-5d\t",  
+	  fprintf(stdout, " %8.2f * %-6.1f\t",  
 		  -cat[i].score/cat[i].complexity,
 		  cat[i].complexity);
 	} else if( u_options & (1<<U_OPTION_POSTERIOR) ) {
@@ -684,9 +827,9 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
 	} else {
 	  fprintf(stdout, 
 		  "%7.2f %7.2f %7.2f %7.2f %8lx\t", 
-		  lambda, ref, -renorm, 
-		  multinomial_correction, 
-		  (long unsigned int)(k ? NTOH_ID(k->id) : 0));
+		  lambda, ref, apply ? -cat[i].renorm : 0.0, 
+		  multinomial_correction,
+		  (long unsigned int)((k && apply) ? NTOH_ID(k->id) : 0));
 	}
       }
 
@@ -694,7 +837,11 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
 
     if( u_options & (1<<U_OPTION_DUMP) ) {
       print_token(stdout, tok);
-      fprintf(stdout, "\n");
+      if( re > 0 ) {
+	fprintf(stdout, "<re=%d>\n", re);
+      } else {
+	fprintf(stdout, "\n");
+      }
     }
 
   }
@@ -731,9 +878,7 @@ confidence_t gamma_pvalue(category_t *cat, double obs) {
  * FILE MANAGEMENT FUNCTIONS                               *
  ***********************************************************/
 
-/* loads a category hash 
-   returns 0 on failure, you should free the category in that case */
-error_code_t load_category(category_t *cat) {
+error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
   hash_count_t i, j;
   char buf[MAGIC_BUFSIZE];
   char scratchbuf[MAGIC_BUFSIZE];
@@ -743,7 +888,14 @@ error_code_t load_category(category_t *cat) {
 
   FILE *input;
 
+  /* this is needed in case we try to open with write permissions,
+     which would otherwise create the file */
+
   input = fopen(cat->fullfilename, "rb");
+  if( input && (strcmp(openf, "rb") != 0) ) {
+    input = freopen(cat->fullfilename, openf, input);
+  }
+
   if( input ) {
     if( !fgets(buf, MAGIC_BUFSIZE, input) ||
 	strncmp(buf, MAGIC1, MAGIC1_LEN) ) {
@@ -764,6 +916,8 @@ error_code_t load_category(category_t *cat) {
       return 0;
     }
     cat->max_order = (token_order_t)shint_val;
+    cat->delta = 1.0/(score_t)(cat->max_order);
+    cat->renorm = cat->delta * cat->logZ;
     if( scratchbuf[0] == 'm' ) {
       cat->model_type = simple;
     } else {
@@ -818,7 +972,7 @@ error_code_t load_category(category_t *cat) {
     /* if this category did not register a regex, it wants
        the default processing, so we flag this */
     if( !cat->retype ) {
-      cat->m_options &= ~(1<<M_OPTION_USE_REGEX);
+      cat->m_options |= (1<<M_OPTION_USE_STDTOK);
     }
 
     /* if we haven't read a character class, use alpha */
@@ -855,58 +1009,9 @@ error_code_t load_category(category_t *cat) {
     }
 #endif
 
-    if( u_options & (1<<U_OPTION_MMAP) ) {
-      cat->mmap_offset = ftell(input);
-      if( cat->mmap_offset > 0 ) {
-	cat->mmap_start = 
-	  (byte_t *)MMAP(0, sizeof(c_item_t) * cat->max_tokens + 
-			 cat->mmap_offset,
-			 PROT_READ, MAP_SHARED, fileno(input), 0);
-	if( cat->mmap_start ) {
-	  cat->hash = (c_item_t *)(cat->mmap_start + cat->mmap_offset);
-	  MADVISE(cat->hash, sizeof(c_item_t) * cat->max_tokens, 
-		  MADV_SEQUENTIAL|MADV_WILLNEED);
-	  /* lock the pages to prevent swapping - on Linux, this
-	     works without root privs so long as the user limits
-	     are big enough - mine are unlimited ;-) 
-	     On other OSes, root may me necessary. If we can't
-	     lock, it doesn't really matter, but cross validations
-             and multiple classifications are a _lot_ faster with locking. */
-	  MLOCK(cat->hash, sizeof(c_item_t) * cat->max_tokens);
-	  cat->c_options |= (1<<C_OPTION_MMAPPED_HASH);
-	}
-      }
-    }
-
-    if( !cat->hash ) {
-      cat->c_options &= ~(1<<C_OPTION_MMAPPED_HASH);
-      /* allocate hash table */
-      cat->hash = (c_item_t *)malloc(sizeof(c_item_t) * cat->max_tokens);
-      if( !cat->hash ) {
-	errormsg(E_ERROR, "not enough memory for category %s\n", 
-		 cat->filename);
-	fclose(input);
-	return 0;
-      }
-
-      MADVISE(cat->hash, sizeof(c_item_t) * cat->max_tokens, 
-	      MADV_SEQUENTIAL);
-
-      /* read in hash table */
-      i = cat->max_tokens;
-      j = 0;
-      while(!ferror(input) && !feof(input) && (j < i) ) {
-	j += fread(cat->hash + j, sizeof(c_item_t), i - j, input);
-      }
-
-      if( j < i ) {
-	errormsg(E_ERROR, "corrupt category? %s\n",
-		 cat->fullfilename);
-	free(cat->hash);
-	fclose(input);
-	return 0;
-      }
-
+    if( !create_category_hash(cat, input, protf) ) {
+      fclose(input);
+      return 0;
     }
 
     fclose(input);
@@ -918,21 +1023,21 @@ error_code_t load_category(category_t *cat) {
 }
 
 
+/* loads a category hash 
+   returns 0 on failure, you should free the category in that case */
+error_code_t load_category(category_t *cat) {
+  return explicit_load_category(cat, "rb", PROT_READ);
+}
+
+/* loads a category file for potential read/write */
+error_code_t open_category(category_t *cat) {
+  return explicit_load_category(cat, "r+b", PROT_READ|PROT_WRITE);
+}
+
 error_code_t reload_category(category_t *cat) {
   if( cat ) {
     /* free the hash, but keep the cat->fullfilename */
-    if( cat->hash ) { 
-      if( cat->c_options & (1<<C_OPTION_MMAPPED_HASH) ) {
-	MUNMAP(cat->mmap_start, cat->max_tokens * sizeof(c_item_t));
-	cat->mmap_start = NULL;
-	cat->hash = NULL;
-      }
-
-      if( cat->hash ) {
-	free(cat->hash);
-	cat->hash = NULL;
-      }
-    }
+    free_category_hash(cat);
     return load_category(cat) && 
       sanitize_model_options(&m_options,cat);
   }
