@@ -32,7 +32,7 @@
 #endif
 #endif
 
-#define COPYBLURB "Copyright (c) 2002-2004 L.A. Breyer\n" \
+#define COPYBLURB "Copyright (c) 2002,2003,2004,2005 L.A. Breyer. All rights reserved.\n" \
                   "%s comes with ABSOLUTELY NO WARRANTY, and is licensed\n" \
 	          "to you under the terms of the GNU General Public License.\n\n"
 
@@ -43,7 +43,15 @@
  * trying to read the category while it is being written.
  */
 #define ATOMIC_CATSAVE
-#define ATOMIC_CREATE(x) open(x, O_CREAT|O_EXCL|O_WRONLY, 0440)
+/* we give our files the 640 permissions - I've added "write"
+   permission because sometimes we want to mmap/readwrite such files
+   so we need those permissions. Also, O_BINARY is not portable, but
+   a good idea for some platforms
+ */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+#define ATOMIC_CREATE(x) open(x, O_CREAT|O_EXCL|O_RDWR|O_BINARY, 0640)
 
 /* we define several memory models, which differ basically 
    in the number of bytes used for the hash tables. Adjust to taste */
@@ -142,6 +150,16 @@ typedef uint64_t u_int64_t;
 #define MMAP(x,y,z,t,u,v) NULL
 #endif
 
+/* constants used by mmap */
+#ifndef PROT_READ
+#define PROT_READ  0
+#define PROT_WRITE 0
+#define PROT_EXEC  0
+#define PROT_NONE  0
+#endif
+
+#define PAGEALIGN(x) ((x) / system_pagesize) * system_pagesize
+
 /* below, FMT_* macros are used in printf/scanf format strings */
 #if defined HUGE_MEMORY_MODEL
 
@@ -426,6 +444,7 @@ typedef u_int16_t digitized_weight_t;
  * MAX_TOKEN_LEN < 1024, which gives MAX_TOKEN_LEN = 30.
  */
 #define MAX_TOKEN_LEN ((charbuf_len_t)30) 
+#define TOKEN_LIST_GROW 1048576L
 
 /* user options */
 #define U_OPTION_CLASSIFY               1
@@ -443,7 +462,7 @@ typedef u_int16_t digitized_weight_t;
 #define U_OPTION_DECIMATE               14
 #define U_OPTION_GROWHASH               15
 #define U_OPTION_INDENTED               16
-#define U_OPTION_ZEROLEARN              17
+#define U_OPTION_NOZEROLEARN            17
 #define U_OPTION_LAPLACE                18
 #define U_OPTION_DIRICHLET              19
 #define U_OPTION_JAYNES                 20
@@ -451,6 +470,8 @@ typedef u_int16_t digitized_weight_t;
 #define U_OPTION_CONFIDENCE             22
 #define U_OPTION_VAR                    23
 
+#define U_OPTION_HM_ADDRESSES           24
+
 /* model options */
 #define M_OPTION_REFMODEL               1
 #define M_OPTION_TEXT_FORMAT            2
@@ -473,7 +494,7 @@ typedef u_int16_t digitized_weight_t;
 #define M_OPTION_CHAR_CEF               20
 #define M_OPTION_SHOW_SCRIPT            21
 #define M_OPTION_SHOW_HTML_COMMENTS     22
-#define M_OPTION_USE_REGEX              23
+#define M_OPTION_USE_STDTOK             23
 #define M_OPTION_ATTACHMENTS            24
 #define M_OPTION_WARNING_BAD            25
 #define M_OPTION_SHOW_STYLE             26
@@ -493,6 +514,7 @@ typedef u_int32_t options_t; /* make sure big enough for all options */
 
 typedef long int re_bitfield;
 #define MAX_RE ((regex_count_t)(8 * sizeof(re_bitfield)))
+#define INVALID_RE 0
 /* maximum number of tagged subexpressions we can handle for each regex */
 #define MAX_SUBMATCH ((token_order_t)9)
 
@@ -676,7 +698,7 @@ typedef struct {
   char *fullfilename;
   mtype model_type;
   token_order_t max_order;
-  token_count_t complexity;
+  token_count_t fcomplexity;
   token_count_t model_unique_token_count;
   token_count_t model_full_token_count;
   document_count_t model_num_docs;
@@ -685,6 +707,9 @@ typedef struct {
   re_bitfield retype;
   score_t logZ;
   score_t divergence;
+  score_t renorm;
+  score_t delta;
+  score_t complexity;
   score_t score;
   score_t score_div;
   score_t score_s2;
@@ -696,9 +721,9 @@ typedef struct {
   score_t s2;
   options_t m_options;
   options_t c_options;
+  c_item_t *hash;
   byte_t *mmap_start;
   long mmap_offset;
-  c_item_t *hash;
 #if defined DIGITIZE_DIGRAMS
   digitized_weight_t dig[ASIZE][ASIZE];
 #else
@@ -741,8 +766,18 @@ typedef struct {
 
 typedef struct {
   char *filename;
-  FILE *tmp;
-  void *tmpiobuf;
+  struct {
+    FILE *file;
+    char *filename;
+    void *iobuf;    
+    long offset;
+    long used;
+    off_t avail;
+    byte_t *mmap_start;
+    size_t mmap_offset;
+    size_t mmap_length;
+    size_t mmap_cursor;
+  } tmp;
   re_bitfield retype;
   token_order_t max_order;
   token_count_t fixed_order_token_count[MAX_SUBMATCH];
@@ -758,6 +793,11 @@ typedef struct {
   score_t beta;
   score_t mu;
   score_t s2;
+  options_t m_options;
+  options_t u_options;
+  byte_t *mmap_start;
+  long mmap_learner_offset;
+  long mmap_hash_offset;
   l_item_t *hash;
   weight_t dig[ASIZE][ASIZE];
   long int regex_token_count[MAX_RE + 1];
@@ -798,7 +838,7 @@ typedef struct {
 
 #define MAX_BOUNDARY_BUFSIZE 70
 
-typedef enum { ceUNDEF, ceID, ceB64, ceQP, ceBIN } MIME_Content_Encoding;
+typedef enum { ceUNDEF, ceID, ceB64, ceQP, ceBIN, ceSEVEN} MIME_Content_Encoding;
 typedef enum { 
   ctUNDEF, 
   ctTEXT_PLAIN, ctTEXT_RICH, ctTEXT_HTML, ctTEXT_XML, ctTEXT_SGML, ctTEXT_UNKNOWN, 
@@ -939,6 +979,7 @@ extern "C"
   c_item_t *find_in_category(category_t *cat, hash_value_t id);
   void init_purely_random_text_category(category_t *cat);
   error_code_t load_category(category_t *cat);
+  error_code_t open_category(category_t *cat);
   void reload_all_categories();
 
   void score_word(char *tok, token_type_t tt, regex_count_t re);
