@@ -348,7 +348,7 @@ void free_mbox_line_filter(MBOX_State *mbox) {
 /* the token class is a common label for a subset of features,
    such as e.g. all features which appear in the header. The label
    should be a number greater than AMIN. If all tokens have the same
-   class, then we effectively obtain he dbacl 1.7 and earlier behaviour. 
+   class, then we effectively obtain the dbacl 1.7 and earlier behaviour. 
    IT DOESN"T MAKE SENSE to have multiple classes and multiple orders.
    It's one or the other, otherwise we need several normalizing constants. 
 */
@@ -441,7 +441,7 @@ void process_directory(char *name,
       if( stat(fullp, &statinfo) == 0 ) {
 	switch(statinfo.st_mode & S_IFMT) {
 	case S_IFREG:
-	  input = fopen(fullp, "r");
+	  input = fopen(fullp, "rb");
 	  if( input ) {
 	    inputfile = fullp;
 	    /* set some initial options */
@@ -466,6 +466,13 @@ void process_directory(char *name,
   }
 }
 
+void reset_current_token(char *tokbuf, char **q, token_order_t *how_many) {
+  tokbuf[0] = DIAMOND;
+  tokbuf[1] = '\0';
+  *q = tokbuf + 1;
+  *how_many = 0;
+}
+
 /* reads a text file as input and applies several filters. */
 void process_file(FILE *input, 
 		  int (*line_filter)(MBOX_State *, char *),
@@ -482,16 +489,16 @@ void process_file(FILE *input,
   int extra_lines = 2;
   e = 0;
   /* initialize the norex state */
-  tokbuf[0] = DIAMOND;
-  tokbuf[1] = '\0';
-  q = tokbuf + 1;
-  how_many = 0;
-
+  reset_current_token(tokbuf, &q, &how_many);
 
   set_iobuf_mode(input);
 
   inputline = 0;
 
+  /* extra lines are used to flush data conversion caches, but not
+     needed for plain text */
+  if( u_options & (1<<U_OPTION_FILTER) ) { extra_lines = 0; }
+
   /* now start processing */
   while( fill_textbuf(input, &extra_lines) ) {
     inputline++;
@@ -523,7 +530,7 @@ void process_file(FILE *input,
 
       /* default processing: reads tokens and passes them to
 	 the word_fun */
-      if( !(m_options & (1<<M_OPTION_USE_REGEX)) ) {
+      if( (m_options & (1<<M_OPTION_USE_STDTOK)) ) {
 	std_tokenizer(pptextbuf, &q, tokbuf, &how_many, ngram_order,
 		      word_fun, get_token_type);
       }
@@ -533,6 +540,10 @@ void process_file(FILE *input,
     /* now summarize this line if required */
     if( post_line_fun ) { (*post_line_fun)(pptextbuf); }
 
+    if( !(m_options & (1<<M_OPTION_NGRAM_STRADDLE_NL)) ) {
+      reset_current_token(tokbuf, &q, &how_many);
+    }
+
     if( cmd & (1<<CMD_RELOAD_CATS) ) {
 
       reload_all_categories();
@@ -542,8 +553,10 @@ void process_file(FILE *input,
 
   }
   /* since std_tokenizer tokens can straddle lines, we should
-     flush the last token fragment */
-  if( !(m_options & (1<<M_OPTION_USE_REGEX)) ) {
+     flush the last token fragment - note this has nothing to do with
+     the M_OPTION_NGRAM_STRADDLE_NL flag, it's an issue caused by caching
+     decoders such as the base64 and qp line filters. */
+  if( (m_options & (1<<M_OPTION_USE_STDTOK)) ) { 
     std_tokenizer(NULL, &q, tokbuf, &how_many, ngram_order,
 		  word_fun, get_token_type);
     if( post_line_fun ) { (*post_line_fun)(NULL); }
@@ -584,7 +597,7 @@ void w_process_directory(char *name,
       if( stat(fullp, &statinfo) == 0 ) {
 	switch(statinfo.st_mode & S_IFMT) {
 	case S_IFREG:
-	  input = fopen(fullp, "r");
+	  input = fopen(fullp, "rb");
 	  if( input ) {
 	    inputfile = fullp;
 	    /* set some initial options */
@@ -631,13 +644,14 @@ void w_process_file(FILE *input,
   set_iobuf_mode(input);
 
   /* initialize the norex state */
-  tokbuf[0] = DIAMOND;
-  tokbuf[1] = '\0';
-  q = tokbuf + 1;
-  how_many = 0;
+  reset_current_token(tokbuf, &q, &how_many);
 
   memset(&input_shiftstate, 0, sizeof(mbstate_t));
   inputline = 0;
+  /* extra lines are used to flush data conversion caches, but not
+     needed for plain text */
+  if( u_options & (1<<U_OPTION_FILTER) ) { extra_lines = 0; }
+
   while( fill_textbuf(input, &extra_lines) ) {
     inputline++;
     /* preprocesses textbuf, optionally censors it */
@@ -673,7 +687,7 @@ void w_process_file(FILE *input,
 
       /* default processing: reads tokens and passes them to
 	 the word_fun */
-      if( !(m_options & (1<<M_OPTION_USE_REGEX)) ) {
+      if( (m_options & (1<<M_OPTION_USE_STDTOK)) ) {
 	w_std_tokenizer(wc_textbuf, &q, tokbuf, &how_many, ngram_order,
 			word_fun, get_token_type);
       }
@@ -683,6 +697,10 @@ void w_process_file(FILE *input,
     /* now summarize this line if required */
     if( post_line_fun ) { (*post_line_fun)(pptextbuf); }
 
+    if( !(m_options & (1<<M_OPTION_NGRAM_STRADDLE_NL)) ) {
+      reset_current_token(tokbuf, &q, &how_many);
+    }
+
     if( cmd & (1<<CMD_RELOAD_CATS) ) {
 
       reload_all_categories();
@@ -693,7 +711,7 @@ void w_process_file(FILE *input,
   }
   /* since w_std_tokenizer tokens can straddle lines, we should
      flush the last token fragment */
-  if( !(m_options & (1<<M_OPTION_USE_REGEX)) ) {
+  if( (m_options & (1<<M_OPTION_USE_STDTOK)) ) {
     w_std_tokenizer(NULL, &q, tokbuf, &how_many, ngram_order,
 		  word_fun, get_token_type);
     if( post_line_fun ) { (*post_line_fun)(NULL); }
