@@ -155,7 +155,7 @@ void load_rc_file() {
   strcat(name, getenv("HOME"));
   strcat(name, "/.mailinspectrc");
 
-  if( (input = fopen(name, "r")) ) {
+  if( (input = fopen(name, "rb")) ) {
     while( !feof(input) ) {
       fgets(buf, PIPE_BUFLEN, input);
       if( buf[0] == 'F' ) {
@@ -213,8 +213,8 @@ void build_scores(weight_t *s) {
 
   if( m_options & (1<<M_OPTION_CALCENTROPY) ) {
     cat[0].score_shannon = 
-      -( cat[0].score_shannon / ((weight_t)cat[0].complexity) -
-	 log((weight_t)cat[0].complexity) );
+      -( cat[0].score_shannon / ((score_t)cat[0].complexity) -
+	 log((score_t)cat[0].complexity) );
     cat[0].score_div = 
       -(cat[0].score/cat[0].complexity + cat[0].score_shannon);
   }
@@ -265,7 +265,8 @@ char *process_email_line(char *textbuf) {
 
       /* reset calculations */
       cat[0].score = 0.0;
-      cat[0].complexity = 0;
+      cat[0].complexity = 0.0;
+      cat[0].fcomplexity = 0;
 
       if( m_options & (1<<M_OPTION_CALCENTROPY) ) {
 	clear_empirical(&empirical);
@@ -352,6 +353,7 @@ void process_last_email() {
 
   cat[0].score = 0.0;
   cat[0].complexity = 0;
+  cat[0].fcomplexity = 0;
   
   if( m_options & (1<<M_OPTION_CALCENTROPY) ) {
     clear_empirical(&empirical);
