@@ -41,3 +41,7 @@ double gamma_tail(double a, double b, double x) {
   /* don't call igamc with extreme numbers, it can exit with an error */
   return igamc(a, x/b);
 }
+
+double normal_cdf(double x) {
+  return ndtr(x);
+}
