@@ -55,6 +55,7 @@ extern void *out_iobuf;
 extern long system_pagesize;
 
 int sa_signal = 0;
+signal_cleanup_t cleanup = { NULL };
 
 /***********************************************************
  * GLOBAL BUFFERS                                          *
@@ -174,13 +175,13 @@ weight_t undigitize_a_weight(digitized_weight_t d, token_order_t order) {
 /***********************************************************
  * SIGNAL HANDLING                                         *
  ***********************************************************/
-#if defined SIGACTION
+#if defined HAVE_SIGACTION
 /* sigaction structure used by several functions */
 struct sigaction act;
 #endif
 
 void my_sa_handler(int signum) {
-#if defined SIGACTION
+#if defined HAVE_SIGACTION
   sa_signal = signum;
 #endif
 }
@@ -193,7 +194,7 @@ void sigsegv(int signum) {
 
 /* intercepts typical termination signals and tries to do the right thing */
 void init_signal_handling() {
-#if defined SIGACTION
+#if defined HAVE_SIGACTION
 
   memset(&act, 0, sizeof(act));
 
@@ -230,20 +231,25 @@ void cleanup_signal_handling() {
 }
 
 void process_pending_signal(FILE *input) {
-#if defined SIGACTION
+#if defined HAVE_SIGACTION
   
   if( sa_signal ) {
 
     sigprocmask(SIG_BLOCK, &act.sa_mask, NULL);
-
     switch(sa_signal) {
     case SIGINT:
+      if( cleanup.tempfile ) { 
+	unlink(cleanup.tempfile);
+      }
       fprintf(stderr, 
 	      "%s:signal: caught interrupt request, exiting\n", progname);
       exit(1);
       break;
     case SIGPIPE:
       /* should we terminate, or should we ignore? */
+      if( cleanup.tempfile ) { 
+	unlink(cleanup.tempfile); 
+      }
       fprintf(stderr,
 	      "%s:error: broken pipe on output, exiting\n", progname);
       exit(1);
