@@ -61,6 +61,10 @@ double normal_cdf(double x);
 extern double igamc(double, double);
 extern double ndtr(double);
 
+typedef struct {
+  const char *tempfile;
+} signal_cleanup_t;
+
 void init_signal_handling();
 void process_pending_signal(FILE *input);
 void cleanup_signal_handling();
